"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AINativeBrowserContribution = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const command_1 = require("@opensumi/ide-core-browser/lib/ai-native/command");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/contextkey/ai-native");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const ide_preferences_1 = require("@opensumi/ide-preferences");
const common_1 = require("../common");
const ai_editor_contribution_1 = require("./ai-editor.contribution");
const ai_native_service_1 = require("./ai-native.service");
const chat_view_1 = require("./chat/chat.view");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const layout_config_1 = require("./layout/layout-config");
const tabbar_view_1 = require("./layout/tabbar.view");
const types_1 = require("./types");
let AINativeBrowserContribution = class AINativeBrowserContribution {
    constructor() {
        this.registerFeature();
    }
    initialize() {
        this.aiNativeConfigService.enableCapabilities();
        const { supportsChatAssistant, supportsOpenSumiDesign } = this.aiNativeConfigService.capabilities;
        const { useMenubarView } = this.aiNativeConfigService.layout;
        let layoutConfig = this.appConfig.layoutConfig;
        let layoutViewSize = this.appConfig.layoutViewSize;
        if (supportsChatAssistant) {
            layoutConfig = Object.assign(Object.assign({}, layoutConfig), layout_config_1.AIChatLayoutConfig);
        }
        if (useMenubarView) {
            layoutViewSize.setMenubarHeight(48);
            layoutConfig = Object.assign(Object.assign({}, layoutConfig), layout_config_1.AIMenubarLayoutConfig);
        }
        if (supportsOpenSumiDesign) {
            layoutViewSize.setEditorTabsHeight(36);
            layoutViewSize.setStatusBarHeight(36);
            layoutViewSize.setAccordionHeaderSizeHeight(36);
        }
        this.appConfig.layoutConfig = layoutConfig;
        this.appConfig.layoutViewSize = layoutViewSize;
    }
    registerFeature() {
        this.contributions.getContributions().forEach((contribution) => {
            if (contribution.registerInlineChatFeature) {
                contribution.registerInlineChatFeature(this.inlineChatFeatureRegistry);
            }
            if (contribution.registerChatFeature) {
                contribution.registerChatFeature(this.chatFeatureRegistry);
            }
            if (contribution.registerResolveConflictFeature) {
                contribution.registerResolveConflictFeature(this.resolveConflictRegistry);
            }
            if (contribution.registerRenameProvider) {
                contribution.registerRenameProvider(this.renameCandidatesProviderRegistry);
            }
        });
    }
    registerSetting(registry) {
        registry.registerSettingGroup({
            id: ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID,
            title: ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID,
            iconClass: (0, ide_core_browser_1.getIcon)('magic-wand'),
        });
        if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
            registry.registerSettingSection(ide_core_browser_1.AI_NATIVE_SETTING_GROUP_ID, {
                title: (0, ide_core_browser_1.localize)('preference.aiNative.inlineChat.title'),
                preferences: [
                    {
                        id: ide_core_browser_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE,
                        localized: 'preference.aiNative.inlineChat.auto.visible',
                    },
                ],
            });
        }
    }
    registerEditorFeature(registry) {
        registry.registerEditorFeatureContribution({
            contribute: (editor) => {
                const aiEditorContribution = this.injector.get(ai_editor_contribution_1.AIEditorContribution, [editor]);
                return aiEditorContribution.contribute(editor);
            },
        });
    }
    registerCommands(commands) {
        commands.registerCommand(command_1.AI_INLINE_CHAT_VISIBLE, {
            execute: (value) => {
                this.aiNativeService.launchInlineChatVisible(value);
            },
        });
        commands.registerCommand(command_1.AI_INLINE_COMPLETION_REPORTER, {
            execute: (relationId, sessionId, accept) => {
                this.aiCompletionsService.report({ sessionId, accept, relationId });
            },
        });
        commands.registerCommand(command_1.AI_INLINE_COMPLETION_VISIBLE, {
            execute: async (visible) => {
                if (!visible) {
                    await this.commandService.executeCommand('editor.action.inlineSuggest.hide');
                    this.aiCompletionsService.hideStatusBarItem();
                    this.aiInlineCompletionsProvider.resetContextKey();
                    this.aiInlineCompletionsProvider.cancelRequest();
                    this.aiCompletionsService.setVisibleCompletion(false);
                }
            },
        });
    }
    registerRenderer(registry) {
        registry.registerSlotRenderer(common_1.AI_CHAT_VIEW_ID, tabbar_view_1.AIChatTabRenderer);
        if (this.aiNativeConfigService.layout.useMergeRightWithLeftPanel) {
            registry.registerSlotRenderer(ide_core_browser_1.SlotLocation.left, tabbar_view_1.AILeftTabRenderer);
            registry.registerSlotRenderer(ide_core_browser_1.SlotLocation.right, tabbar_view_1.AIRightTabRenderer);
        }
    }
    registerComponent(registry) {
        registry.register(common_1.AI_CHAT_CONTAINER_ID, [], {
            component: chat_view_1.AIChatView,
            containerId: common_1.AI_CHAT_CONTAINER_ID,
        });
    }
    registerKeybindings(keybindings) {
        if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
            keybindings.registerKeybinding({
                command: command_1.AI_INLINE_CHAT_VISIBLE.id,
                keybinding: 'ctrlcmd+i',
                when: 'editorTextFocus',
                args: true,
                priority: 0,
            }, ide_core_browser_1.KeybindingScope.USER);
            keybindings.registerKeybinding({
                command: command_1.AI_INLINE_CHAT_VISIBLE.id,
                keybinding: 'esc',
                args: false,
                when: `editorFocus && ${ai_native_1.InlineChatIsVisible.raw}`,
            });
        }
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AINativeBrowserContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AppConfig),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "appConfig", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AINativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "chatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ResolveConflictRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "resolveConflictRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.RenameCandidatesProviderRegistryToken),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "renameCandidatesProviderRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_native_service_1.AINativeService),
    tslib_1.__metadata("design:type", ai_native_service_1.AINativeService)
], AINativeBrowserContribution.prototype, "aiNativeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], AINativeBrowserContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AINativeBrowserContribution.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AIInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AIInlineCompletionsProvider)
], AINativeBrowserContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.CommandService),
    tslib_1.__metadata("design:type", Object)
], AINativeBrowserContribution.prototype, "commandService", void 0);
AINativeBrowserContribution = tslib_1.__decorate([
    (0, ide_core_browser_1.Domain)(ide_core_browser_1.ClientAppContribution, browser_1.BrowserEditorContribution, ide_core_browser_1.CommandContribution, ide_preferences_1.SettingContribution, ide_core_browser_1.KeybindingContribution, ide_core_browser_1.ComponentContribution, ide_core_browser_1.SlotRendererContribution),
    tslib_1.__metadata("design:paramtypes", [])
], AINativeBrowserContribution);
exports.AINativeBrowserContribution = AINativeBrowserContribution;
//# sourceMappingURL=ai-core.contribution.js.map