"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIEditorContribution = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-core-browser/lib/menu/next/renderer/ctxmenu/browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const menu_service_1 = require("@opensumi/ide-design/lib/browser/override/menu.service");
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const common_1 = require("../common");
const ai_native_service_1 = require("./ai-native.service");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const rename_service_1 = require("./rename/rename.service");
const types_1 = require("./types");
const inline_chat_feature_registry_1 = require("./widget/inline-chat/inline-chat.feature.registry");
const inline_chat_service_1 = require("./widget/inline-chat/inline-chat.service");
const inline_content_widget_1 = require("./widget/inline-chat/inline-content-widget");
const inline_diff_widget_1 = require("./widget/inline-diff/inline-diff-widget");
let AIEditorContribution = class AIEditorContribution extends ide_core_common_1.Disposable {
    constructor() {
        super();
        this.aiInlineChatDisposed = new ide_core_common_1.Disposable();
        this.aiInlineChatOperationDisposed = new ide_core_common_1.Disposable();
        this.logger = this.loggerManagerClient.getLogger(ide_core_common_1.SupportLogNamespace.Browser);
    }
    disposeAllWidget() {
        [
            this.aiDiffWidget,
            this.aiInlineContentWidget,
            this.aiInlineChatDisposed,
            this.aiInlineChatOperationDisposed,
        ].forEach((widget) => {
            widget === null || widget === void 0 ? void 0 : widget.dispose();
        });
        this.aiNativeService.cancelToken();
    }
    contribute(editor) {
        if (!editor) {
            return this;
        }
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return this;
        }
        if (this.aiNativeConfigService.capabilities.supportsInlineCompletion) {
            this.contributions.getContributions().forEach((contribution) => {
                if (contribution.middleware) {
                    this.latestMiddlewareCollector = contribution.middleware;
                }
            });
            this.registerCompletion(editor);
        }
        if (this.aiNativeConfigService.capabilities.supportsRenameSuggestions) {
            this.registerRenameSuggestions(editor);
        }
        this.disposables.push(monacoEditor.onWillChangeModel(() => {
            this.disposeAllWidget();
        }));
        this.disposables.push(monacoEditor.onDidScrollChange(() => {
            /**
             * 其他的 ctxmenu 服务注册的菜单在 onHide 函数里会有其他逻辑处理，例如在 editor.context.ts 会在 hide 的时候 focus 编辑器，影响使用
             */
            this.ctxMenuRenderer.onHide = undefined;
            this.ctxMenuRenderer.hide(true);
        }));
        this.disposables.push(this.aiNativeService.onInlineChatVisible((value) => {
            if (value) {
                this.registerInlineChat(editor);
            }
            else {
                this.disposeAllWidget();
            }
        }));
        let isShowInlineChat = false;
        this.disposables.push(monacoEditor.onMouseDown(() => {
            isShowInlineChat = false;
        }), monacoEditor.onMouseUp((event) => {
            const target = event.target;
            const detail = target.detail;
            if (detail && typeof detail === 'string' && detail === common_1.AIInlineChatContentWidget) {
                isShowInlineChat = false;
            }
            else {
                isShowInlineChat = true;
            }
        }));
        ide_core_common_1.Event.debounce(ide_core_common_1.Event.any(monacoEditor.onDidChangeCursorSelection, monacoEditor.onMouseUp), (_, e) => e, 100)(() => {
            if (!this.preferenceService.getValid(ide_core_common_1.AINativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE)) {
                return;
            }
            if (!isShowInlineChat) {
                return;
            }
            if (this.aiInlineChatService.status !== 0 /* EInlineChatStatus.READY */ &&
                this.aiInlineChatService.status !== 3 /* EInlineChatStatus.ERROR */) {
                return;
            }
            this.registerInlineChat(editor);
        });
        return this;
    }
    async registerInlineChat(editor) {
        var _a;
        if (!this.aiNativeConfigService.capabilities.supportsInlineChat) {
            return;
        }
        this.disposeAllWidget();
        const { monacoEditor, currentUri } = editor;
        if (!currentUri || currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        const selection = monacoEditor.getSelection();
        const selectCode = selection && ((_a = monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.getValueInRange(selection));
        if (!selection || !(selectCode === null || selectCode === void 0 ? void 0 : selectCode.trim())) {
            this.disposeAllWidget();
            return;
        }
        this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
        this.aiInlineContentWidget = this.injector.get(inline_content_widget_1.AIInlineContentWidget, [monacoEditor]);
        this.aiInlineContentWidget.show({
            selection,
        });
        this.aiInlineChatDisposed.addDispose(this.aiInlineContentWidget.onClickActions(async (id) => {
            const handler = this.inlineChatFeatureRegistry.getEditorHandler(id);
            const action = this.inlineChatFeatureRegistry.getAction(id);
            if (!handler || !action) {
                return;
            }
            const { execute, providerDiffPreviewStrategy } = handler;
            if (execute) {
                execute(monacoEditor);
                this.disposeAllWidget();
            }
            if (providerDiffPreviewStrategy) {
                const crossSelection = selection
                    .setStartPosition(selection.startLineNumber, 1)
                    .setEndPosition(selection.endLineNumber, Number.MAX_SAFE_INTEGER);
                await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection);
                this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onDiscard(() => {
                    this.disposeAllWidget();
                }));
                this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onRegenerate(async () => {
                    await this.handleDiffPreviewStrategy(monacoEditor, providerDiffPreviewStrategy, crossSelection);
                }));
            }
        }));
    }
    async handleDiffPreviewStrategy(monacoEditor, strategy, crossSelection) {
        const model = monacoEditor.getModel();
        if (!model || !crossSelection) {
            return;
        }
        this.resetDiffEnvironment();
        const crossCode = model.getValueInRange(crossSelection);
        this.aiInlineChatService.launchChatStatus(1 /* EInlineChatStatus.THINKING */);
        const response = await strategy(monacoEditor, this.aiNativeService.cancelIndicator.token);
        if (this.aiInlineChatDisposed.disposed || ide_core_common_1.CancelResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
            return;
        }
        if (ide_core_common_1.ErrorResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(3 /* EInlineChatStatus.ERROR */);
            return;
        }
        this.aiInlineChatService.launchChatStatus(2 /* EInlineChatStatus.DONE */);
        let answer = this.extractAnswerFromResponse(response);
        if (!answer) {
            return;
        }
        answer = this.formatAnswer(answer, crossCode);
        this.visibleDiffWidget(monacoEditor, crossSelection, answer);
        this.aiInlineChatOperationDisposed.addDispose([
            this.aiInlineChatService.onAccept(() => {
                var _a;
                (_a = monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.pushEditOperations(null, [{ range: crossSelection, text: answer }], () => null);
                (0, ide_core_common_1.runWhenIdle)(() => {
                    this.disposeAllWidget();
                });
            }),
            this.aiDiffWidget.onMaxLincCount((count) => {
                requestAnimationFrame(() => {
                    if (crossSelection.endLineNumber === model.getLineCount()) {
                        const lineHeight = monacoEditor.getOption(monaco_api_1.monaco.editor.EditorOption.lineHeight);
                        this.aiInlineContentWidget.offsetTop(lineHeight * count + 12);
                    }
                });
            }),
        ]);
        return answer;
    }
    resetDiffEnvironment() {
        var _a;
        (_a = this.aiDiffWidget) === null || _a === void 0 ? void 0 : _a.dispose();
        this.aiInlineChatOperationDisposed.dispose();
    }
    extractAnswerFromResponse(response) {
        const regex = /```\w*([\s\S]+?)\s*```/;
        const match = regex.exec(response.message);
        return match ? match[1].trim() : response.message.trim();
    }
    formatAnswer(answer, crossCode) {
        const leadingWhitespaceMatch = crossCode.match(/^\s*/);
        const indent = leadingWhitespaceMatch ? leadingWhitespaceMatch[0] : '  ';
        return answer
            .split('\n')
            .map((line, index) => (index === 0 ? line : `${indent}${line}`))
            .join('\n');
    }
    visibleDiffWidget(monacoEditor, crossSelection, answer) {
        monacoEditor.setHiddenAreas([crossSelection], inline_diff_widget_1.AIDiffWidget._hideId);
        this.aiDiffWidget = this.injector.get(inline_diff_widget_1.AIDiffWidget, [monacoEditor, crossSelection, answer]);
        this.aiDiffWidget.create();
        this.aiDiffWidget.showByLine(crossSelection.startLineNumber - 1, crossSelection.endLineNumber - crossSelection.startLineNumber + 2);
        this.updateInlineContentWidgetPosition(crossSelection);
    }
    updateInlineContentWidgetPosition(crossSelection) {
        var _a, _b;
        (_a = this.aiInlineContentWidget) === null || _a === void 0 ? void 0 : _a.setOptions({
            position: {
                lineNumber: crossSelection.endLineNumber + 1,
                column: 1,
            },
        });
        (_b = this.aiInlineContentWidget) === null || _b === void 0 ? void 0 : _b.layoutContentWidget();
    }
    /**
     * 代码补全
     */
    async registerCompletion(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        let dispose;
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async (event) => {
            if (dispose) {
                dispose.dispose();
                this.aiInlineCompletionsProvider.dispose();
            }
            const { monacoEditor, currentUri } = editor;
            if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
                return this;
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            // 取光标的当前位置
            const position = monacoEditor.getPosition();
            if (!position) {
                return;
            }
            this.aiInlineCompletionsProvider.registerEditor(editor);
            dispose = monaco_api_1.monaco.languages.registerInlineCompletionsProvider(model.getLanguageId(), {
                provideInlineCompletions: async (model, position, context, token) => {
                    var _a, _b, _c, _d;
                    if ((_b = (_a = this.latestMiddlewareCollector) === null || _a === void 0 ? void 0 : _a.language) === null || _b === void 0 ? void 0 : _b.provideInlineCompletions) {
                        this.aiCompletionsService.setMiddlewareComplete((_d = (_c = this.latestMiddlewareCollector) === null || _c === void 0 ? void 0 : _c.language) === null || _d === void 0 ? void 0 : _d.provideInlineCompletions);
                    }
                    const list = await this.aiInlineCompletionsProvider.provideInlineCompletionItems(model, position, context, token);
                    this.logger.log('provideInlineCompletions: ', list.items.map((data) => data.insertText));
                    return list;
                },
                freeInlineCompletions(completions) { },
                handleItemDidShow: (completions, item) => {
                    if (completions.items.length > 0) {
                        this.aiCompletionsService.setVisibleCompletion(true);
                    }
                },
            });
            this.disposables.push(dispose);
        }));
        return;
    }
    async registerRenameSuggestions(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        let dispose;
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async (event) => {
            if (dispose) {
                dispose.dispose();
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            const provider = async (model, range, token) => {
                const result = await this.renameSuggestionService.provideRenameSuggestions(model, range, token);
                return result;
            };
            dispose = monaco_api_1.monaco.languages.registerNewSymbolNameProvider(model.getLanguageId(), {
                provideNewSymbolNames: provider,
            });
        }));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIEditorContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AINativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AINativeConfigService)
], AIEditorContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AIInlineChatService)
], AIEditorContribution.prototype, "aiInlineChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_native_service_1.AINativeService),
    tslib_1.__metadata("design:type", ai_native_service_1.AINativeService)
], AIEditorContribution.prototype, "aiNativeService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ILoggerManagerClient),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "loggerManagerClient", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.IBrowserCtxMenu),
    tslib_1.__metadata("design:type", menu_service_1.DesignBrowserCtxMenuService)
], AIEditorContribution.prototype, "ctxMenuRenderer", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.InlineChatFeatureRegistryToken),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AIEditorContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AINativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AIEditorContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AIInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AIInlineCompletionsProvider)
], AIEditorContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(rename_service_1.RenameSuggestionsService),
    tslib_1.__metadata("design:type", rename_service_1.RenameSuggestionsService)
], AIEditorContribution.prototype, "renameSuggestionService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AIEditorContribution.prototype, "aiCompletionsService", void 0);
AIEditorContribution = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], AIEditorContribution);
exports.AIEditorContribution = AIEditorContribution;
//# sourceMappingURL=ai-editor.contribution.js.map