"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PS1TerminalService = void 0;
const tslib_1 = require("tslib");
const dom_align_1 = tslib_1.__importDefault(require("dom-align"));
const react_1 = tslib_1.__importDefault(require("react"));
const client_1 = require("react-dom/client");
const uuid_1 = require("uuid");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_terminal_next_1 = require("@opensumi/ide-terminal-next");
const chat_service_1 = require("../chat/chat.service");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const terminal_command_suggest_controller_1 = require("./component/terminal-command-suggest-controller");
// 基于 PS1 Hack 的终端 AI 能力集成
var IstermOscPt;
(function (IstermOscPt) {
    IstermOscPt["PromptStarted"] = "PS";
    IstermOscPt["PromptEnded"] = "PE";
    IstermOscPt["CurrentWorkingDirectory"] = "CWD";
})(IstermOscPt || (IstermOscPt = {}));
let PS1TerminalService = class PS1TerminalService extends ide_core_common_1.Disposable {
    active() {
        this.initContainer();
        this.disposables.push(this.terminalController.onDidOpenTerminal(({ id }) => this.listenTerminalEvent(id)));
    }
    listenTerminalEvent(clientId) {
        const client = this.terminalController.clients.get(clientId);
        if (client) {
            setTimeout(() => {
                this.listenPromptState(client.term);
            }, 0);
        }
    }
    initContainer() {
        this.popupContainer = document.createElement('div');
        document.body.appendChild(this.popupContainer);
    }
    listenPromptState(xterm) {
        xterm.parser.registerOscHandler(6973, (data) => {
            const argsIndex = data.indexOf(';');
            const sequence = argsIndex === -1 ? data : data.substring(0, argsIndex);
            switch (sequence) {
                case IstermOscPt.PromptEnded:
                    this.handlePromptEnd(xterm);
                    break;
                default:
                    return false;
            }
            // 不要阻止事件传递
            return false;
        });
    }
    handlePromptEnd(xterm) {
        if (this.promptEndMarker) {
            this.promptEndMarker.dispose();
        }
        if (this.promptEndDecoration) {
            this.promptEndDecoration.dispose();
        }
        this.promptEndMarker = xterm.registerMarker(0);
        const xOffset2 = xterm.buffer.active.cursorX;
        if (this.onDataDisposable) {
            this.onDataDisposable.dispose();
        }
        const aiHinitDecoration = xterm.registerDecoration({
            marker: this.promptEndMarker,
            width: xterm.cols,
            height: 1,
            x: xOffset2 + 2,
            layer: 'top',
        });
        aiHinitDecoration === null || aiHinitDecoration === void 0 ? void 0 : aiHinitDecoration.onRender((element) => {
            element.innerText = (0, ide_core_browser_1.localize)('terminal.ai.inputSharpToGetHint');
            element.style.opacity = '0.3';
            // 提示框点击也可以触发 AI 弹窗
            element.onclick = () => {
                this.showAICommandPopup(xterm, xOffset2);
            };
        });
        this.onDataDisposable = xterm.onData((e) => {
            var _a;
            // 获取当前活动缓冲区
            const buffer = xterm.buffer.active;
            // 获取光标位置
            const cursorX = buffer.cursorX;
            const cursorY = buffer.cursorY;
            // 用户有输入时，就隐藏掉提示框
            aiHinitDecoration === null || aiHinitDecoration === void 0 ? void 0 : aiHinitDecoration.dispose();
            if (e === '#' &&
                cursorY + buffer.baseY === ((_a = this.promptEndMarker) === null || _a === void 0 ? void 0 : _a.line) &&
                (cursorX === xOffset2 || cursorX === xOffset2 + 1)) {
                this.showAICommandPopup(xterm, xOffset2);
            }
        });
    }
    /**
     * 展示 AI 提示命令的弹框
     * @param xterm Xterm 实例
     * @param xOffset2 传入的 X 偏移坐标
     * @returns
     */
    showAICommandPopup(xterm, xOffset2) {
        var _a, _b, _c;
        if (!this.promptEndMarker) {
            return;
        }
        this.promptEndDecoration = xterm.registerDecoration({
            marker: this.promptEndMarker,
            width: 1,
            height: 1,
            backgroundColor: '#2472C8',
            x: xOffset2,
            layer: 'top',
        });
        // HACK: 这里拿去 TerminalConnection 的方式很 Hack，看看有没有更好的办法？
        // @ts-ignore
        const connectionWrapper = xterm._addonManager._addons.find((addon) => { var _a; return !!((_a = addon === null || addon === void 0 ? void 0 : addon.instance) === null || _a === void 0 ? void 0 : _a.connection); });
        const connection = (_a = connectionWrapper === null || connectionWrapper === void 0 ? void 0 : connectionWrapper.instance) === null || _a === void 0 ? void 0 : _a.connection;
        if (!connection) {
            // eslint-disable-next-line no-console
            console.error((0, ide_core_browser_1.localize)('terminal.ai.cannotGetTerminalConnection'));
            return;
        }
        (_b = this.promptEndDecoration) === null || _b === void 0 ? void 0 : _b.onRender((element) => {
            const domWidth = element.clientWidth;
            if (this.promptEndReactRoot) {
                this.promptEndReactRoot.unmount();
            }
            this.promptEndReactRoot = (0, client_1.createRoot)(this.popupContainer);
            // 直接在 Decoration DOM 上渲染 input 输入框的话，input 的事件会被 xterm 影响导致无法聚焦
            // 因此采用外层 DOM + DOMAlign + 生命周期绑定 的方式来渲染输入框和 AI 提示框
            this.promptEndReactRoot.render(react_1.default.createElement(terminal_command_suggest_controller_1.AITerminalPrompt, { onEscTriggered: () => {
                    setTimeout(() => {
                        var _a;
                        connection.sendData('\b');
                        (_a = this.promptEndDecoration) === null || _a === void 0 ? void 0 : _a.dispose();
                    }, 0);
                }, onSuggestionClick: (suggestion) => {
                    setTimeout(() => {
                        var _a;
                        connection.sendData('\b');
                        connection.sendData(suggestion);
                        (_a = this.promptEndDecoration) === null || _a === void 0 ? void 0 : _a.dispose();
                    }, 0);
                }, getAICommandSuggestions: this.getAICommandSuggestions.bind(this), cancelAIRequst: this.stopAIStreamRequest.bind(this) }));
            (0, dom_align_1.default)(this.popupContainer, element, {
                points: ['bl', 'bl'],
                offset: [-domWidth - 6, 0],
                targetOffset: [0, 0],
                overflow: { adjustX: false, adjustY: false },
            });
        });
        // AI 输入自然语言命令的输入框被取消，需要后续处理的逻辑
        (_c = this.promptEndDecoration) === null || _c === void 0 ? void 0 : _c.onDispose(() => {
            this.stopAIStreamRequest();
            // 取消 AI 悬浮框的 React 渲染
            if (this.promptEndReactRoot) {
                this.promptEndReactRoot.unmount();
            }
            // 聚焦焦点到 Xterm 上
            xterm.focus();
        });
    }
    async stopAIStreamRequest() {
        if (this.currentSessionId) {
            await this.aiChatService.destroyStreamRequest(this.currentSessionId);
            this.aiChatService.cancelChatViewToken();
        }
    }
    // 获取 AI 命令建议
    async getAICommandSuggestions(commandDescription, statCallback, suggestionCallback) {
        // 根据国际化配置调整 AI Prompt 的提示语
        const aiPrompt = `${(0, ide_core_browser_1.localize)('terminal.ai.modelPrompt')} ${commandDescription}`;
        // 如果已有 AI 请求发出，那就取消这个 AI 请求
        await this.stopAIStreamRequest();
        const reportRelationId = this.aiReporter.start('terminalAICommand', { message: commandDescription });
        const sessionId = (0, uuid_1.v4)();
        await this.aiChatService.messageWithStream(aiPrompt, {}, sessionId);
        this.currentSessionId = sessionId;
        let buffer = ''; // 用于累积数据片段的缓冲区
        const aiCommandSuggestions = [];
        let currentObj = {};
        // 流式解析大模型返回的每行数据
        const processLine = (lineBuffer) => {
            const firstCommandIndex = lineBuffer.indexOf('#Command#:');
            let line = lineBuffer;
            if (firstCommandIndex !== -1) {
                // 找到了第一个#Command#:，截取它及之后的内容
                line = lineBuffer.substring(firstCommandIndex);
            }
            // 解析命令和描述
            if (line.startsWith('#Command#:')) {
                if (currentObj.command) {
                    // 如果currentObj中已有命令，则将其添加到结果数组中，并开始新的对象
                    currentObj = {};
                }
                currentObj.command = line.substring('#Command#:'.length).trim();
            }
            else if (line.startsWith('#Description#:')) {
                currentObj.description = line.substring('#Description#:'.length).trim();
                aiCommandSuggestions.push(currentObj);
                if (aiCommandSuggestions.length > 4) {
                    // 如果 AI 返回的命令超过 5 个，就停止 AI 生成 (这种情况下往往是模型不稳定或者出现了幻觉)
                    this.stopAIStreamRequest();
                }
                suggestionCallback(aiCommandSuggestions); // 每拿到一个结果就回调一次，优化用户体感
            }
        };
        // 大模型 Stream 流式返回监听，缓存处理，然后按行解析
        this.msgStreamManager.onMsgListChange(sessionId)((msg) => {
            if (msg && this.msgStreamManager.currentSessionId === sessionId) {
                const { delta, finish_reason } = msg;
                const chunk = delta.content;
                buffer += chunk;
                if (finish_reason === 'stop') {
                    buffer += '\n';
                }
                let newlineIndex = buffer.indexOf('\n');
                while (newlineIndex !== -1) {
                    const line = buffer.substring(0, newlineIndex).trim();
                    buffer = buffer.substring(newlineIndex + 1);
                    processLine(line);
                    newlineIndex = buffer.indexOf('\n');
                }
            }
        });
        // 大模型 Stream 状态监听
        this.msgStreamManager.onMsgStatus((status) => {
            if (this.msgStreamManager.currentSessionId === sessionId) {
                statCallback(status);
                if (status === 2) {
                    // AI 思考完成
                    this.aiReporter.end(reportRelationId, { message: commandDescription, success: true });
                    this.currentSessionId = '';
                }
            }
        });
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_terminal_next_1.ITerminalController),
    tslib_1.__metadata("design:type", Object)
], PS1TerminalService.prototype, "terminalController", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(chat_service_1.ChatService),
    tslib_1.__metadata("design:type", chat_service_1.ChatService)
], PS1TerminalService.prototype, "aiChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(msg_stream_manager_1.MsgStreamManager),
    tslib_1.__metadata("design:type", msg_stream_manager_1.MsgStreamManager)
], PS1TerminalService.prototype, "msgStreamManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], PS1TerminalService.prototype, "aiReporter", void 0);
PS1TerminalService = tslib_1.__decorate([
    (0, di_1.Injectable)()
], PS1TerminalService);
exports.PS1TerminalService = PS1TerminalService;
//# sourceMappingURL=ps1-terminal.service.js.map