"use strict";
var _ChatService_sessionModel;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatService = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const common_1 = require("../../common");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const chat_manager_service_1 = require("./chat-manager.service");
let ChatService = class ChatService extends ide_core_common_1.Disposable {
    get latestSessionId() {
        return this._latestSessionId;
    }
    get sessionModel() {
        return tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f");
    }
    constructor() {
        super();
        this._onChatMessageLaunch = new ide_core_common_1.Emitter();
        this.onChatMessageLaunch = this._onChatMessageLaunch.event;
        this._onChangeSessionId = new ide_core_common_1.Emitter();
        this.onChangeSessionId = this._onChangeSessionId.event;
        _ChatService_sessionModel.set(this, void 0);
        this.cancelIndicatorChatView = new ide_core_common_1.CancellationTokenSource();
        tslib_1.__classPrivateFieldSet(this, _ChatService_sessionModel, this.chatManagerService.startSession(), "f");
    }
    launchChatMessage(data) {
        this._onChatMessageLaunch.fire(data);
    }
    cancelChatViewToken() {
        this.cancelIndicatorChatView.cancel();
        this.cancelIndicatorChatView = new ide_core_common_1.CancellationTokenSource();
    }
    async destroyStreamRequest(sessionId) {
        if (this.aiBackService.destroyStreamRequest) {
            await this.aiBackService.destroyStreamRequest(sessionId);
            this.msgStreamManager.sendDoneStatue();
        }
    }
    /**
     * by backend service
     * @param message
     */
    onMessage(message, sessionId) {
        try {
            const msgObj = JSON.parse(message);
            if (msgObj.id && msgObj.choices) {
                const { id, choices } = msgObj;
                this.msgStreamManager.recordMessage(id, choices[0]);
            }
            else {
                this.msgStreamManager.sendErrorStatue();
            }
            if (msgObj.errorCode && sessionId) {
                const errMsg = msgObj.errorMsg || '';
                this.aiReporter.end(sessionId, {
                    message: errMsg,
                    success: false,
                });
            }
        }
        catch (error) {
            throw new Error(`onMessage error: ${error}`);
        }
    }
    async messageWithStream(input, options = {}, sessionId) {
        this.msgStreamManager.setCurrentSessionId(sessionId);
        this.msgStreamManager.sendThinkingStatue();
        await this.aiBackService.requestStream(input, Object.assign(Object.assign({}, options), { sessionId }), this.cancelIndicatorChatView.token);
    }
    async message(input, options = {}) {
        const res = await this.aiBackService.request(input, options, this.cancelIndicatorChatView.token);
        if (res.isCancel) {
            return null;
        }
        if (res.errorCode !== 0) {
            return res.errorMsg || '';
        }
        else {
            return res.data || '';
        }
    }
    setLatestSessionId(id) {
        this._latestSessionId = id;
        this._onChangeSessionId.fire(id);
    }
    createRequest(input, agentId, command) {
        return this.chatManagerService.createRequest(tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f").sessionId, input, agentId, command);
    }
    sendRequest(request, regenerate = false) {
        return this.chatManagerService.sendRequest(tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f").sessionId, request, regenerate);
    }
    cancelRequest() {
        this.chatManagerService.cancelRequest(tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f").sessionId);
    }
    clearSessionModel() {
        this.chatManagerService.clearSession(tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f").sessionId);
        tslib_1.__classPrivateFieldSet(this, _ChatService_sessionModel, this.chatManagerService.startSession(), "f");
    }
    dispose() {
        var _a;
        (_a = tslib_1.__classPrivateFieldGet(this, _ChatService_sessionModel, "f")) === null || _a === void 0 ? void 0 : _a.dispose();
        super.dispose();
    }
};
_ChatService_sessionModel = new WeakMap();
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.AIBackSerivcePath),
    tslib_1.__metadata("design:type", Object)
], ChatService.prototype, "aiBackService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], ChatService.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(msg_stream_manager_1.MsgStreamManager),
    tslib_1.__metadata("design:type", msg_stream_manager_1.MsgStreamManager)
], ChatService.prototype, "msgStreamManager", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], ChatService.prototype, "aiReporter", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(common_1.IChatManagerService),
    tslib_1.__metadata("design:type", chat_manager_service_1.ChatManagerService)
], ChatService.prototype, "chatManagerService", void 0);
ChatService = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], ChatService);
exports.ChatService = ChatService;
//# sourceMappingURL=chat.service.js.map