"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChatNotify = exports.ChatReply = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const button_1 = require("@opensumi/ide-components/lib/button");
const recycle_tree_1 = require("@opensumi/ide-components/lib/recycle-tree");
const tree_node_define_1 = require("@opensumi/ide-components/lib/recycle-tree/basic/tree-node.define");
const tooltip_1 = require("@opensumi/ide-components/lib/tooltip");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_theme_1 = require("@opensumi/ide-theme");
const htmlContent_1 = require("@opensumi/monaco-editor-core/esm/vs/base/common/htmlContent");
const common_1 = require("../../common");
const msg_stream_manager_1 = require("../model/msg-stream-manager");
const ChatMarkdown_1 = require("./ChatMarkdown");
const components_module_less_1 = tslib_1.__importDefault(require("./components.module.less"));
const Loading_1 = require("./Loading");
const Thinking_1 = require("./Thinking");
const TreeRenderer = (props) => {
    const labelService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.LabelService);
    const commandService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.CommandService);
    const getIconClassName = (uri, isDirectory, expanded) => {
        // getIcon 没有处理 isOpenedDirectory
        let iconClassName = labelService.getIcon(uri, { isDirectory });
        if (isDirectory && expanded) {
            iconClassName += ' expanded';
        }
        return iconClassName;
    };
    const recycleTreeData = (0, react_1.useMemo)(() => {
        const transform = (item) => {
            const isDirectory = typeof item.type === 'number' ? item.type === ide_core_common_1.FileType.Directory : !!item.children;
            const uri = new ide_core_common_1.URI(item.uri);
            return {
                label: item.label,
                iconClassName: getIconClassName(uri, isDirectory, isDirectory),
                expandable: true,
                expanded: true,
                children: isDirectory ? (item.children || []).map(transform) : null,
                uri,
            };
        };
        return (props.treeData.children || []).map(transform);
    }, [props.treeData]);
    const [height, setHeight] = (0, react_1.useState)(22);
    const fileHandle = (0, react_1.useRef)(null);
    const onReady = (handle) => {
        fileHandle.current = handle;
        const calcHeight = () => {
            let size = handle.getModel().root.branchSize;
            if (size < 1) {
                size = 1;
            }
            else if (size > 20) {
                size = 20;
            }
            setHeight(size * 22);
        };
        calcHeight();
        handle.onDidUpdate(calcHeight);
    };
    if (!recycleTreeData.length) {
        return null;
    }
    return (react_1.default.createElement("div", { className: components_module_less_1.default.tree_container },
        react_1.default.createElement(recycle_tree_1.BasicRecycleTree, { height: height, treeData: recycleTreeData, onContextMenu: (e) => e.preventDefault(), onClick: (e, item) => {
                if (!fileHandle.current || !item) {
                    return;
                }
                if (!tree_node_define_1.BasicCompositeTreeNode.is(item)) {
                    commandService.executeCommand(ide_core_browser_1.EDITOR_COMMANDS.OPEN_RESOURCE.id, item.raw.uri, {
                        disableNavigate: true,
                        preview: true,
                    });
                }
                else {
                    item.raw.iconClassName = getIconClassName(item.raw.uri, true, item.expanded);
                }
            }, onReady: onReady, treeName: props.treeData.label, leaveBottomBlank: false, baseIndent: 0 })));
};
const ComponentRender = (props) => {
    const chatAgentViewService = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.ChatAgentViewServiceToken);
    const [node, setNode] = (0, react_1.useState)(null);
    (0, react_1.useEffect)(() => {
        const config = chatAgentViewService.getChatComponent(props.component);
        if (config) {
            const { component: Component, initialProps } = config;
            setNode(react_1.default.createElement(Component, Object.assign({}, initialProps, { value: props.value })));
            return;
        }
        setNode(react_1.default.createElement("div", null,
            react_1.default.createElement(Loading_1.Loading, null),
            react_1.default.createElement("span", { style: { marginLeft: 4 } }, "\u6B63\u5728\u52A0\u8F7D\u7EC4\u4EF6")));
        const deferred = chatAgentViewService.getChatComponentDeferred(props.component);
        deferred.promise.then(({ component: Component, initialProps }) => {
            setNode(react_1.default.createElement(Component, Object.assign({}, initialProps, { value: props.value })));
        });
    }, []);
    return node;
};
const ChatReply = (props) => {
    var _a, _b;
    const { relationId, request, startTime = 0, onRegenerate } = props;
    const [, update] = (0, react_1.useReducer)((num) => (num + 1) % 1000000, 0);
    const msgStreamManager = (0, ide_core_browser_1.useInjectable)(msg_stream_manager_1.MsgStreamManager);
    const aiReporter = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.IAIReporter);
    const iconService = (0, ide_core_browser_1.useInjectable)(ide_theme_1.IIconService);
    const contextKeyService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.IContextKeyService);
    const aiChatService = (0, ide_core_browser_1.useInjectable)(common_1.IAIChatService);
    const chatAgentService = (0, ide_core_browser_1.useInjectable)(common_1.IChatAgentService);
    const isLastReply = msgStreamManager.currentSessionId === relationId;
    (0, react_1.useEffect)(() => {
        const disposableCollection = new ide_core_browser_1.DisposableCollection();
        disposableCollection.push(request.response.onDidChange(() => {
            if (request.response.isComplete) {
                aiReporter.end(relationId, {
                    message: request.response.responseText,
                    replytime: Date.now() - startTime,
                    success: true,
                    isStop: false,
                });
                msgStreamManager.sendDoneStatue();
            }
            else {
                msgStreamManager.sendThinkingStatue();
            }
            update();
        }));
        disposableCollection.push(msgStreamManager.onMsgStatus(() => {
            if (msgStreamManager.currentSessionId !== relationId) {
                update();
            }
        }));
        return () => disposableCollection.dispose();
    }, [relationId]);
    const handleRegenerate = (0, react_1.useCallback)(() => {
        request.response.reset();
        onRegenerate === null || onRegenerate === void 0 ? void 0 : onRegenerate();
    }, [onRegenerate]);
    const onStop = () => {
        aiReporter.end(relationId, {
            message: request.response.responseText,
            replytime: Date.now() - startTime,
            success: false,
            isStop: true,
        });
        aiChatService.cancelRequest();
    };
    const renderMarkdown = (markdown) => (
    // TODO: Markdown 属性限制
    react_1.default.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: markdown, fillInIncompleteTokens: true }));
    const renderTreeData = (treeData) => react_1.default.createElement(TreeRenderer, { treeData: treeData });
    const renderPlaceholder = (markdown) => (react_1.default.createElement("div", { className: components_module_less_1.default.placeholder },
        react_1.default.createElement(components_1.Icon, { iconClass: iconService.fromString('$(codicon/sync~spin)') }),
        react_1.default.createElement("div", { className: components_module_less_1.default.placeholder_content }, renderMarkdown(markdown))));
    const renderComponent = (componentId, value) => (react_1.default.createElement(ComponentRender, { component: componentId, value: value }));
    const contentNode = react_1.default.useMemo(() => request.response.responseContents.map((item, index) => {
        let node;
        if (item.kind === 'asyncContent') {
            node = renderPlaceholder(new htmlContent_1.MarkdownString(item.content));
        }
        else if (item.kind === 'treeData') {
            node = renderTreeData(item.treeData);
        }
        else if (item.kind === 'component') {
            node = renderComponent(item.component, item.value);
        }
        else {
            node = renderMarkdown(item.content);
        }
        return react_1.default.createElement(react_1.Fragment, { key: `${item.kind}-${index}` }, node);
    }), [request.response.responseContents]);
    const followupNode = react_1.default.useMemo(() => {
        if (!request.response.followups) {
            return null;
        }
        return request.response.followups.map((item, index) => {
            let node = null;
            if (item.kind === 'reply') {
                const a = (react_1.default.createElement("a", { onClick: () => aiChatService.launchChatMessage(chatAgentService.parseMessage(item.message)) }, item.title || item.message));
                node = item.tooltip ? react_1.default.createElement(tooltip_1.Tooltip, { title: item.tooltip }, a) : a;
            }
            else {
                if (item.when && !contextKeyService.match(item.when)) {
                    node = null;
                }
                node = react_1.default.createElement(button_1.Button, { type: 'default' }, item.title);
            }
            return node && react_1.default.createElement(react_1.Fragment, { key: index }, node);
        });
    }, [request.response.followups]);
    if (!request.response.isComplete && isLastReply) {
        return (react_1.default.createElement(Thinking_1.Thinking, { status: msgStreamManager.status, message: request.response.responseText, onStop: onStop, hasAgent: true }, contentNode));
    }
    return (react_1.default.createElement(Thinking_1.ThinkingResult, { status: 2 /* EMsgStreamStatus.DONE */, hasMessage: request.response.responseParts.length > 0 || !!((_a = request.response.errorDetails) === null || _a === void 0 ? void 0 : _a.message), onRegenerate: handleRegenerate, sessionId: relationId },
        react_1.default.createElement("div", { className: components_module_less_1.default.ai_chat_response_container }, ((_b = request.response.errorDetails) === null || _b === void 0 ? void 0 : _b.message) ? (react_1.default.createElement("div", { className: components_module_less_1.default.error },
            react_1.default.createElement(components_1.Icon, { className: (0, components_1.getIcon)('close-circle') }),
            react_1.default.createElement("span", null, request.response.errorDetails.message))) : (react_1.default.createElement(react_1.default.Fragment, null,
            contentNode,
            (followupNode === null || followupNode === void 0 ? void 0 : followupNode.length) !== 0 && react_1.default.createElement("div", { className: components_module_less_1.default.followups }, followupNode))))));
};
exports.ChatReply = ChatReply;
const ChatNotify = (props) => (react_1.default.createElement(Thinking_1.ThinkingResult, { status: 2 /* EMsgStreamStatus.DONE */, hasMessage: true, sessionId: props.relationId, regenerateDisabled: true },
    react_1.default.createElement(ChatMarkdown_1.ChatMarkdown, { markdown: props.chunk.content, relationId: props.relationId })));
exports.ChatNotify = ChatNotify;
//# sourceMappingURL=ChatReply.js.map