import { WithEventBus } from '@opensumi/ide-core-common';
import { IAICompletionResultModel } from '@opensumi/ide-core-common/lib/types/ai-native';
import { IAIReporter } from '@opensumi/ide-core-common/lib/types/ai-native/reporter';
import { IEditor } from '@opensumi/ide-editor';
import * as monaco from '@opensumi/ide-monaco';
import { InlineCompletionItem } from './model/competionModel';
import { PromptCache } from './promptCache';
import { AICompletionsService } from './service/ai-completions.service';
export declare const getAcceptText: (cuont: number) => void;
declare class RequestImp {
    model: monaco.editor.ITextModel;
    _isManual: boolean;
    aiCompletionsService: AICompletionsService;
    promptCache: PromptCache;
    isCancelFlag: boolean;
    constructor(model: monaco.editor.ITextModel, _isManual: boolean, aiCompletionsService: AICompletionsService, promptCache: PromptCache);
    sendRequest(position: monaco.Position, aiReporter: IAIReporter, model: monaco.editor.ITextModel, token: monaco.CancellationToken): Promise<InlineCompletionItem[]>;
    /**
     * 将补全结果推给用户并注册{@link COMMAND_ACCEPT} 事件
     * @param codeModelList
     */
    pushResultAndRegist(rs: IAICompletionResultModel, position: monaco.Position, relationId: string): Array<InlineCompletionItem>;
    cancelRequest(): void;
}
declare class ReqStack {
    private readonly aiReporter;
    queue: any[];
    constructor(aiReporter: any);
    addReq(reqRequest: RequestImp): void;
    runReq(position: monaco.Position, model: monaco.editor.ITextModel, token: monaco.CancellationToken): any;
    cancleRqe(): void;
}
/**
 * 代码补全provider
 * @param extensionContext
 * @param this.tyStatusBarItem
 * @returns
 */
interface ProviderType extends monaco.languages.InlineCompletionsProvider {
    isManual: boolean;
}
export declare class AIInlineCompletionsProvider extends WithEventBus implements ProviderType {
    private aiCompletionsService;
    private readonly injector;
    private aiReporter;
    private aiNativeContextKey;
    private editor;
    private promptCache;
    isManual: boolean;
    isDelEvent: boolean;
    reqStack: ReqStack;
    registerEditor(editor: IEditor): void;
    provideInlineCompletions(model: monaco.editor.ITextModel, position: monaco.Position, context: monaco.languages.InlineCompletionContext, token: monaco.CancellationToken): monaco.languages.ProviderResult<monaco.languages.InlineCompletions<monaco.languages.InlineCompletion>>;
    handleItemDidShow?(completions: monaco.languages.InlineCompletions<monaco.languages.InlineCompletion>, item: monaco.languages.InlineCompletion): void;
    freeInlineCompletions(completions: monaco.languages.InlineCompletions<monaco.languages.InlineCompletion>): void;
    cancelRequest(): void;
    resetContextKey(): void;
    /**
     * 用户触发编辑器后，事件的回调
     * @param document
     * @param position
     * @param context
     * @param token
     * @returns
     */
    provideInlineCompletionItems(model: monaco.editor.ITextModel, position: monaco.Position, context: monaco.languages.InlineCompletionContext, token: monaco.CancellationToken): Promise<any>;
    /**
     * 更新isManual的值，判断是否是主动补全
     */
    udateIsManualVal(val: boolean): void;
}
export {};
//# sourceMappingURL=completeProvider.d.ts.map