"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIInlineCompletionsProvider = exports.getAcceptText = void 0;
const tslib_1 = require("tslib");
const lodash_1 = require("lodash");
const di_1 = require("@opensumi/di");
const command_1 = require("@opensumi/ide-core-browser/lib/ai-native/command");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const reporter_1 = require("@opensumi/ide-core-common/lib/types/ai-native/reporter");
const browser_1 = require("@opensumi/ide-editor/lib/browser");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const ai_native_contextkey_service_1 = require("../contextkey/ai-native.contextkey.service");
const promptCache_1 = require("./promptCache");
const provider_1 = require("./provider");
const ai_completions_service_1 = require("./service/ai-completions.service");
let lastRequestId;
let timer;
let accept = 0;
// 用来缓存最近一次的补全结果
const lastInLayList = {
    line: -1,
    column: -1,
    lastResult: null,
};
// 修改埋点代码数据
const getAcceptText = (cuont) => {
    accept = accept + cuont;
};
exports.getAcceptText = getAcceptText;
class RequestImp {
    // todo
    constructor(model, _isManual, aiCompletionsService, promptCache) {
        this.model = model;
        this._isManual = _isManual;
        this.aiCompletionsService = aiCompletionsService;
        this.promptCache = promptCache;
        this.isCancelFlag = false;
    }
    // 发送请求
    async sendRequest(position, aiReporter, model, token) {
        const { model: editor } = this;
        const beginRequestTime = Date.now();
        if (!editor) {
            return [];
        }
        const startRange = new monaco.Range(0, 0, position.lineNumber, position.column);
        let prompt = model.getValueInRange(startRange);
        // 如果是空白页面,默认加个回车符?
        if (this.model.getValue() === '') {
            prompt += '\n';
        }
        const endRange = new monaco.Range(position.lineNumber, position.column, model.getLineCount(), Number.MAX_SAFE_INTEGER);
        let suffix = model.getValueInRange(endRange);
        // 改用analysisCodeFuseLanguage  现在目前后端需要语言小写
        const languageId = model.getLanguageId();
        // if (completionRtModel !== undefined && completionRtModel.sessionId === null) {
        //     //此处未采纳上一个补全结果,那么设置endtime,并提交到服务端
        //     completionRtModel.endRenderingTime = Date.now();
        //     submitCompleteRtData(completionRtModel);
        // }
        // TODO 缺少数据前处理
        // prompt = prePromptHandler(prompt);
        // suffix = preSuffixHandler(suffix);
        prompt = (0, provider_1.prePromptHandler)(prompt);
        suffix = (0, provider_1.preSuffixHandler)(suffix);
        // 组装请求参数,向远程发起请求
        const completionRequestBean = {
            prompt,
            suffix,
            sessionId: (0, ide_core_common_1.uuid)(6),
            language: languageId,
            fileUrl: model.uri.toString().split('/').pop(),
        };
        this.aiCompletionsService.updateStatusBarItem('running', true);
        const beginAlgTime = +new Date();
        let status = 0; // 0: 远程请求的结果 1: 网络缓存中的结果
        if (this.isCancelFlag) {
            return [];
        }
        let rs;
        const cacheData = this.promptCache.getCache(prompt);
        const relationId = aiReporter.start(reporter_1.AISerivceType.Completion, { message: reporter_1.AISerivceType.Completion });
        this.aiCompletionsService.setLastRelationId(relationId);
        // 如果存在缓存
        if (cacheData) {
            rs = cacheData;
            status = 1;
        }
        else {
            try {
                rs = await this.aiCompletionsService.complete(completionRequestBean, model, position, token);
            }
            catch (error) {
                this.aiCompletionsService.reporterEnd(relationId, {
                    success: false,
                    replytime: +new Date() - beginAlgTime,
                    message: error.toString(),
                });
                this.aiCompletionsService.hideStatusBarItem();
                return [];
            }
            status = 0;
        }
        if (!(rs && rs.sessionId)) {
            this.aiCompletionsService.reporterEnd(relationId, { success: false, replytime: +new Date() - beginAlgTime });
            this.aiCompletionsService.hideStatusBarItem();
            return [];
        }
        this.aiCompletionsService.setLastSessionId(rs.sessionId);
        // 如果是取消直接返回
        if ((rs && rs.isCancel) || this.isCancelFlag) {
            this.aiCompletionsService.reporterEnd(relationId, {
                success: true,
                replytime: +new Date() - beginAlgTime,
                isStop: true,
                completionNum: 0,
            });
            this.aiCompletionsService.updateStatusBarItem('canceled', false);
            return [];
        }
        if (rs && rs.codeModelList && rs.codeModelList.length > 0) {
            this.promptCache.setCache(prompt, rs);
        }
        let codeModelSize = 0;
        if (rs.codeModelList !== null) {
            codeModelSize = rs.codeModelList.length;
        }
        // 返回补全结果为空直接返回
        if (rs.codeModelList.length === 0) {
            this.aiCompletionsService.reporterEnd(relationId, {
                success: true,
                replytime: +new Date() - beginAlgTime,
                completionNum: 0,
            });
            this.aiCompletionsService.updateStatusBarItem('no result', false);
            return [];
        }
        this.aiCompletionsService.updateStatusBarItem('completion result: ' + rs.codeModelList.length, false);
        return this.pushResultAndRegist(rs, position, relationId);
    }
    /**
     * 将补全结果推给用户并注册{@link COMMAND_ACCEPT} 事件
     * @param codeModelList
     */
    pushResultAndRegist(rs, position, relationId) {
        const result = new Array();
        for (const codeModel of rs.codeModelList) {
            const contentText = codeModel.content;
            let str = contentText;
            while (true) {
                if (!str.endsWith('\n')) {
                    break;
                }
                str = str.slice(0, -1);
            }
            const insertText = str;
            const model = this.model;
            const textAfterCursor = model.getValueInRange({
                startLineNumber: position.lineNumber,
                startColumn: position.column,
                endLineNumber: position.lineNumber,
                endColumn: model.getLineMaxColumn(position.lineNumber),
            });
            // 临时修复方案，用于解决补全后面多了几个括号的问题
            const removeChars = (a, b) => {
                let result = '';
                for (const char of b) {
                    if (char === ' ' || !a.includes(char)) {
                        result += char;
                    }
                }
                return result;
            };
            const filteredString = removeChars(insertText, textAfterCursor);
            result.push({
                insertText: insertText + filteredString,
                range: new monaco.Range(position.lineNumber, position.column, position.lineNumber, position.column + insertText.length + textAfterCursor.length),
                sessionId: rs.sessionId,
                command: {
                    id: command_1.AI_INLINE_COMPLETION_REPORTER.id,
                    title: '',
                    arguments: [relationId, rs.sessionId, true],
                },
            });
        }
        lastRequestId = rs.sessionId;
        return result;
    }
    cancelRequest() {
        this.isCancelFlag = true;
    }
}
class ReqStack {
    constructor(aiReporter) {
        this.aiReporter = aiReporter;
        this.queue = [];
    }
    addReq(reqRequest) {
        this.queue.push(reqRequest);
    }
    runReq(position, model, token) {
        if (this.queue.length === 0) {
            return;
        }
        const fn = this.queue.pop();
        return fn.sendRequest(position, this.aiReporter, model, token);
    }
    cancleRqe() {
        if (this.queue.length === 0) {
            return;
        }
        this.queue.forEach((item) => {
            item.cancelRequest();
        });
        this.queue = [];
    }
}
let AIInlineCompletionsProvider = class AIInlineCompletionsProvider extends ide_core_common_1.WithEventBus {
    registerEditor(editor) {
        this.aiNativeContextKey = this.injector.get(ai_native_contextkey_service_1.AINativeContextKey, [editor.monacoEditor._contextKeyService]);
        this.editor = editor;
        this.isManual = false;
        this.isDelEvent = false;
        this.reqStack = new ReqStack(this.aiReporter);
        // 判断用户是否选择了一块区域或者移动光标 取消掉请补全求
        const selectionChange = () => {
            // clearPromptMessage()
            const editor = this.editor;
            if (!editor) {
                return;
            }
            // clearPromptMessage();
            this.aiCompletionsService.hideStatusBarItem();
            const selection = editor.monacoEditor.getSelection();
            // 判断是否选中区域
            if (selection.startLineNumber !== selection.endLineNumber || selection.startColumn !== selection.endColumn) {
                this.cancelRequest();
            }
            requestAnimationFrame(() => {
                this.aiCompletionsService.setVisibleCompletion(false);
            });
        };
        const debouncedSelectionChange = (0, lodash_1.debounce)(() => selectionChange(), 50, {
            maxWait: 200,
            leading: true,
            trailing: true,
        });
        this.addDispose(this.eventBus.on(browser_1.EditorSelectionChangeEvent, (e) => {
            if (e.payload.source === 'mouse') {
                debouncedSelectionChange();
            }
            else {
                debouncedSelectionChange.cancel();
                selectionChange();
            }
        }));
        this.addDispose(this.editor.monacoEditor.onDidChangeModelContent((e) => {
            const changes = e.changes;
            for (const change of changes) {
                if (change.text === '') {
                    this.isDelEvent = true;
                    this.cancelRequest();
                }
                else {
                    this.isDelEvent = false;
                }
            }
        }));
        this.addDispose(this.editor.monacoEditor.onWillChangeModel(() => {
            this.aiCompletionsService.hideStatusBarItem();
        }));
        this.addDispose(this.editor.monacoEditor.onDidBlurEditorText(() => {
            this.aiCompletionsService.hideStatusBarItem();
            this.aiCompletionsService.setVisibleCompletion(false);
        }));
    }
    provideInlineCompletions(model, position, context, token) {
        throw new Error('Method not implemented.');
    }
    handleItemDidShow(completions, item) {
        throw new Error('Method not implemented.');
    }
    freeInlineCompletions(completions) {
        throw new Error('Method not implemented.');
    }
    // 取消请求
    cancelRequest() {
        this.aiCompletionsService.cancelRequest();
        if (this.reqStack) {
            this.reqStack.cancleRqe();
        }
        if (timer) {
            clearTimeout(timer);
        }
    }
    resetContextKey() {
        this.aiNativeContextKey.inlineCompletionIsTrigger.reset();
    }
    /**
     * 用户触发编辑器后，事件的回调
     * @param document
     * @param position
     * @param context
     * @param token
     * @returns
     */
    async provideInlineCompletionItems(model, position, context, token) {
        this.aiNativeContextKey.inlineCompletionIsTrigger.set(true);
        // bugfix:修复当鼠标移动到代码补全上会触发一次手势事件，增加防抖，当手势触发后，能够防抖一次
        if (context.triggerKind === 0) {
            if (lastInLayList.column === position.column &&
                lastInLayList.line === position.lineNumber &&
                lastInLayList.lastResult !== undefined) {
                lastInLayList.column = -1;
                lastInLayList.line = -1;
                return lastInLayList.lastResult;
            }
        }
        this.cancelRequest();
        this.aiCompletionsService.hideStatusBarItem();
        // step 1 判断生成开关,如果关闭不进行后续操作
        const _isManual = this.isManual;
        if (this.isDelEvent && !_isManual) {
            return {
                items: [],
            };
        }
        // 重置防止不触发自动补全事件
        this.udateIsManualVal(false);
        // 如果用户已取消
        if (token === null || token === void 0 ? void 0 : token.isCancellationRequested) {
            this.aiCompletionsService.updateStatusBarItem('completion not avaliable ', false);
            return {
                items: [],
            };
        }
        // 放入队列
        const requestImp = new RequestImp(model, _isManual, this.aiCompletionsService, this.promptCache);
        this.reqStack.addReq(requestImp);
        // 如果是自动补全等待300ms
        if (!_isManual) {
            await new Promise((f) => {
                timer = setTimeout(f, 300);
            });
        }
        const list = await this.reqStack.runReq(position, model, token);
        if (position !== undefined) {
            lastInLayList.column = position.column;
            lastInLayList.line = position.lineNumber;
        }
        lastInLayList.lastResult = {
            items: list || [],
        };
        return {
            items: list || [],
        };
    }
    /**
     * 更新isManual的值，判断是否是主动补全
     */
    udateIsManualVal(val) {
        this.isManual = val;
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AICompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AICompletionsService)
], AIInlineCompletionsProvider.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIInlineCompletionsProvider.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(reporter_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], AIInlineCompletionsProvider.prototype, "aiReporter", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", promptCache_1.PromptCache)
], AIInlineCompletionsProvider.prototype, "promptCache", void 0);
AIInlineCompletionsProvider = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: false })
], AIInlineCompletionsProvider);
exports.AIInlineCompletionsProvider = AIInlineCompletionsProvider;
//# sourceMappingURL=completeProvider.js.map