"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIMenuBarView = void 0;
const tslib_1 = require("tslib");
const classnames_1 = tslib_1.__importDefault(require("classnames"));
const React = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const components_1 = require("@opensumi/ide-core-browser/lib/components");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const view_id_1 = require("@opensumi/ide-core-browser/lib/layout/view-id");
const next_1 = require("@opensumi/ide-core-browser/lib/menu/next");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const ide_main_layout_1 = require("@opensumi/ide-main-layout");
const common_1 = require("../../../common/");
const layout_config_1 = require("../layout-config");
const logo_svg_1 = tslib_1.__importDefault(require("./logo.svg"));
const menu_bar_module_less_1 = tslib_1.__importDefault(require("./menu-bar.module.less"));
const AIMenuBarRender = () => {
    const contextmenuService = (0, ide_core_browser_1.useInjectable)(next_1.AbstractContextMenuService);
    const ctxMenuRenderer = (0, ide_core_browser_1.useInjectable)(next_1.ICtxMenuRenderer);
    const aiNativeConfigService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.AINativeConfigService);
    const iconRef = React.useRef(null);
    const [anchor, setAnchor] = React.useState(undefined);
    React.useEffect(() => {
        if (iconRef.current) {
            const rect = iconRef.current.getBoundingClientRect();
            const { x, y, height } = rect;
            setAnchor({
                x,
                y: y + height + 4,
            });
        }
    }, []);
    const extraTopMenus = React.useMemo(() => contextmenuService.createMenu({
        id: next_1.MenuId.AIMenuBarTopExtra,
    }), [contextmenuService]);
    const handleClick = React.useCallback(() => {
        if (!anchor) {
            return;
        }
        const menuNodes = extraTopMenus.getMergedMenuNodes();
        extraTopMenus.dispose();
        ctxMenuRenderer.show({
            anchor,
            menuNodes,
        });
    }, [anchor, extraTopMenus]);
    const logo = React.useMemo(() => aiNativeConfigService.layout.menubarLogo || logo_svg_1.default, [aiNativeConfigService.layout.menubarLogo]);
    return (React.createElement(React.Fragment, null,
        React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: menu_bar_module_less_1.default.ai_enhance_menu, ref: iconRef, onClick: handleClick },
            React.createElement("div", { className: menu_bar_module_less_1.default.logo_container },
                React.createElement("img", { className: menu_bar_module_less_1.default.extra_top_icon, src: logo, alt: '' }),
                React.createElement(components_1.Icon, { className: (0, classnames_1.default)((0, ide_core_browser_1.getIcon)('down'), menu_bar_module_less_1.default.caret_icon) })))));
};
const AIMenuBarView = () => {
    var _a;
    const commandService = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.CommandService);
    const mainLayoutService = (0, ide_core_browser_1.useInjectable)(ide_main_layout_1.IMainLayoutService);
    const aiNativeConfigService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.AINativeConfigService);
    const layoutService = (0, ide_core_browser_1.useInjectable)(ide_main_layout_1.IMainLayoutService);
    const [isVisiablePanel, setIsVisiablePanel] = React.useState(false);
    React.useEffect(() => {
        requestAnimationFrame(() => {
            setIsVisiablePanel(isVisiable());
        });
    }, []);
    const handleLeftMenuVisiable = React.useCallback(() => {
        commandService.executeCommand('main-layout.left-panel.toggle');
        requestAnimationFrame(() => {
            setIsVisiablePanel(isVisiable());
        });
    }, []);
    const isVisiable = React.useCallback(() => {
        const tabbarService = mainLayoutService.getTabbarService(ide_core_browser_1.SlotLocation.left);
        return !!tabbarService.currentContainerId;
    }, [mainLayoutService]);
    const handleChatVisible = React.useCallback(() => {
        layoutService.toggleSlot(common_1.AI_CHAT_VIEW_ID);
    }, [layoutService]);
    return (React.createElement("div", { id: view_id_1.VIEW_CONTAINERS.MENUBAR, className: menu_bar_module_less_1.default.menu_bar_view, style: { height: (_a = aiNativeConfigService.appConfig.layoutViewSize) === null || _a === void 0 ? void 0 : _a.menubarHeight } },
        React.createElement("div", { className: menu_bar_module_less_1.default.container },
            React.createElement("div", { className: menu_bar_module_less_1.default.left },
                React.createElement(ai_native_1.EnhanceIcon, { wrapperClassName: menu_bar_module_less_1.default.enhance_menu, icon: isVisiablePanel ? 'left-nav-open' : 'left-nav-close', onClick: handleLeftMenuVisiable }),
                React.createElement("span", { className: menu_bar_module_less_1.default.dividing }),
                React.createElement("div", { className: menu_bar_module_less_1.default.top_menus_bar },
                    React.createElement(AIMenuBarRender, null)),
                React.createElement(ide_core_browser_1.SlotRenderer, { slot: layout_config_1.AI_MENU_BAR_LEFT, flex: 1, overflow: 'initial' })),
            React.createElement("div", { className: menu_bar_module_less_1.default.right },
                React.createElement(ide_core_browser_1.SlotRenderer, { slot: layout_config_1.AI_MENU_BAR_RIGHT, flex: 1, overflow: 'initial' }),
                React.createElement("div", { className: menu_bar_module_less_1.default.ai_switch, onClick: handleChatVisible },
                    React.createElement(ai_native_1.AILogoAvatar, { iconClassName: menu_bar_module_less_1.default.avatar_icon_large }))))));
};
exports.AIMenuBarView = AIMenuBarView;
//# sourceMappingURL=menu-bar.view.js.map