"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIInlineChatController = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ai_native_1 = require("@opensumi/ide-core-browser/lib/components/ai-native");
const containerPanel_1 = require("@opensumi/ide-core-browser/lib/components/ai-native/content-widget/containerPanel");
const base_1 = require("@opensumi/ide-core-browser/lib/menu/next/base");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const Loading_1 = require("../../components/Loading");
const inline_chat_module_less_1 = tslib_1.__importDefault(require("./inline-chat.module.less"));
const AIInlineOperation = (props) => {
    const { hanldeActions, onClose } = props;
    const inlineChatFeatureRegistry = (0, ide_core_browser_1.useInjectable)(ide_core_common_1.InlineChatFeatureRegistryToken);
    const operationList = (0, react_1.useMemo)(() => inlineChatFeatureRegistry.getEditorActionButtons(), [inlineChatFeatureRegistry]);
    const handleClickActions = (0, react_1.useCallback)((id) => {
        hanldeActions(id);
    }, [hanldeActions]);
    const handleClose = (0, react_1.useCallback)(() => {
        if (onClose) {
            onClose();
        }
    }, [onClose]);
    const moreOperation = (0, react_1.useMemo)(() => inlineChatFeatureRegistry.getEditorActionMenus().map((data) => new base_1.MenuNode({
        id: `ai.menu.operation.${data.id}`,
        label: data.name,
        className: inline_chat_module_less_1.default.more_operation_menu_item,
        execute: () => {
            handleClickActions(data.id);
        },
    })), [inlineChatFeatureRegistry]);
    if (operationList.length === 0 && moreOperation.length === 0) {
        return null;
    }
    return (react_1.default.createElement(ai_native_1.AIAction, { operationList: operationList, moreOperation: moreOperation, onClickItem: handleClickActions, onClose: handleClose }));
};
const AIInlineChatController = (props) => {
    const { onClickActions, onClose } = props;
    const aiInlineChatService = (0, ide_core_browser_1.useInjectable)(ide_core_browser_1.IAIInlineChatService);
    const [status, setStatus] = (0, react_1.useState)(0 /* EInlineChatStatus.READY */);
    (0, react_1.useEffect)(() => {
        const dis = aiInlineChatService.onChatStatus((status) => {
            setStatus(status);
        });
        return () => {
            dis.dispose();
        };
    }, []);
    (0, react_1.useEffect)(() => {
        if (status === 3 /* EInlineChatStatus.ERROR */) {
            if (onClose) {
                onClose();
            }
        }
    }, [status, onClose]);
    const isLoading = (0, react_1.useMemo)(() => status === 1 /* EInlineChatStatus.THINKING */, [status]);
    const isDone = (0, react_1.useMemo)(() => status === 2 /* EInlineChatStatus.DONE */, [status]);
    const isError = (0, react_1.useMemo)(() => status === 3 /* EInlineChatStatus.ERROR */, [status]);
    const iconResultItems = (0, react_1.useMemo)(() => [
        {
            icon: 'check',
            text: (0, ide_core_common_1.localize)('aiNative.inline.chat.operate.check.title'),
            onClick: () => {
                aiInlineChatService._onAccept.fire();
            },
        },
        {
            icon: 'discard',
            text: (0, ide_core_common_1.localize)('aiNative.operate.discard.title'),
            onClick: () => {
                aiInlineChatService._onDiscard.fire();
            },
        },
        {
            icon: 'afresh',
            text: (0, ide_core_common_1.localize)('aiNative.operate.afresh.title'),
            onClick: () => {
                aiInlineChatService._onRegenerate.fire();
            },
        },
    ], []);
    const handleClickActions = (0, react_1.useCallback)((id) => {
        if (onClickActions) {
            onClickActions.fire(id);
        }
    }, [onClickActions]);
    const translateY = (0, react_1.useMemo)(() => {
        if (isDone) {
            return {
                transform: 'translateY(-15px)',
            };
        }
        return undefined;
    }, [isDone]);
    const renderContent = (0, react_1.useCallback)(() => {
        if (isError) {
            return null;
        }
        if (isDone) {
            return (react_1.default.createElement(containerPanel_1.ContentWidgetContainerPanel, { style: { transform: 'translateY(4px)' } },
                react_1.default.createElement(ai_native_1.AIInlineResult, { iconItems: iconResultItems })));
        }
        if (isLoading) {
            return (react_1.default.createElement(containerPanel_1.ContentWidgetContainerPanel, null,
                react_1.default.createElement(ai_native_1.EnhancePopover, { id: 'inline_chat_loading', title: (0, ide_core_common_1.localize)('aiNative.inline.chat.operate.loading.cancel') },
                    react_1.default.createElement(Loading_1.Loading, { className: inline_chat_module_less_1.default.ai_inline_chat_loading }))));
        }
        return react_1.default.createElement(AIInlineOperation, { hanldeActions: handleClickActions, onClose: onClose });
    }, [status]);
    return react_1.default.createElement("div", { style: translateY }, renderContent());
};
exports.AIInlineChatController = AIInlineChatController;
//# sourceMappingURL=inline-chat-controller.js.map