"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InlineChatFeatureRegistry = void 0;
const tslib_1 = require("tslib");
const di_1 = require("@opensumi/di");
const ide_core_common_1 = require("@opensumi/ide-core-common");
let InlineChatFeatureRegistry = class InlineChatFeatureRegistry extends ide_core_common_1.Disposable {
    constructor() {
        super(...arguments);
        this.logger = (0, ide_core_common_1.getDebugLogger)();
        this.actionsMap = new Map();
        this.editorHandlerMap = new Map();
        this.terminalHandlerMap = new Map();
    }
    dispose() {
        super.dispose();
        this.actionsMap.clear();
        this.editorHandlerMap.clear();
    }
    collectActions(operational) {
        const { id } = operational;
        if (this.actionsMap.has(id)) {
            this.logger.warn(`InlineChatFeatureRegistry: id ${id} already exists`);
            return false;
        }
        this.actionsMap.set(id, operational);
        return true;
    }
    registerEditorInlineChat(operational, handler) {
        const isCollect = this.collectActions(operational);
        if (isCollect) {
            this.editorHandlerMap.set(operational.id, handler);
        }
    }
    registerTerminalInlineChat(operational, handler) {
        const isCollect = this.collectActions(operational);
        if (isCollect) {
            if ((0, ide_core_common_1.isUndefined)(handler.triggerRules)) {
                handler.triggerRules = 'selection';
            }
            this.terminalHandlerMap.set(operational.id, handler);
        }
    }
    getEditorActionButtons() {
        return Array.from(this.editorHandlerMap.keys())
            .filter((id) => {
            const actions = this.actionsMap.get(id);
            return actions && actions.renderType === 'button';
        })
            .map((id) => this.actionsMap.get(id));
    }
    getEditorActionMenus() {
        return Array.from(this.editorHandlerMap.keys())
            .filter((id) => {
            const actions = this.actionsMap.get(id);
            return actions && actions.renderType === 'dropdown';
        })
            .map((id) => this.actionsMap.get(id));
    }
    getEditorHandler(id) {
        return this.editorHandlerMap.get(id);
    }
    getTerminalHandler(id) {
        return this.terminalHandlerMap.get(id);
    }
    getTerminalActions() {
        return Array.from(this.terminalHandlerMap.keys()).map((id) => this.actionsMap.get(id));
    }
    getAction(id) {
        return this.actionsMap.get(id);
    }
};
InlineChatFeatureRegistry = tslib_1.__decorate([
    (0, di_1.Injectable)()
], InlineChatFeatureRegistry);
exports.InlineChatFeatureRegistry = InlineChatFeatureRegistry;
//# sourceMappingURL=inline-chat.feature.registry.js.map