"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AIInlineContentWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const browser_1 = require("@opensumi/ide-monaco/lib/browser");
const BaseInlineContentWidget_1 = require("@opensumi/ide-monaco/lib/browser/ai-native/BaseInlineContentWidget");
const index_1 = require("../../../common/index");
const ai_native_contextkey_service_1 = require("../../contextkey/ai-native.contextkey.service");
const inline_chat_controller_1 = require("./inline-chat-controller");
const inline_chat_service_1 = require("./inline-chat.service");
let AIInlineContentWidget = class AIInlineContentWidget extends BaseInlineContentWidget_1.BaseInlineContentWidget {
    constructor(editor) {
        super(editor);
        this.editor = editor;
        this.originTop = 0;
        this._onClickActions = new ide_core_common_1.Emitter();
        this.onClickActions = this._onClickActions.event;
        this.aiNativeContextKey = this.injector.get(ai_native_contextkey_service_1.AINativeContextKey, [this.editor._contextKeyService]);
        this.addDispose(this.editor.onDidLayoutChange(() => {
            if (this.isOutOfArea()) {
                this.dispose();
            }
        }));
    }
    dispose() {
        this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
        super.dispose();
    }
    renderView() {
        return react_1.default.createElement(inline_chat_controller_1.AIInlineChatController, { onClickActions: this._onClickActions, onClose: () => this.dispose() });
    }
    async show(options) {
        super.show(options);
        this.aiNativeContextKey.inlineChatIsVisible.set(true);
    }
    getDomNode() {
        const domNode = super.getDomNode();
        domNode.style.padding = '6px';
        domNode.style.zIndex = '999';
        domNode.style.paddingRight = '50px';
        return domNode;
    }
    async hide(options) {
        this.aiNativeContextKey.inlineChatIsVisible.set(false);
        super.hide();
    }
    /**
     * 如果编辑器区域宽度小于 270px，则不显示
     * 不包括左侧 content width 和右侧的 minimap width
     */
    isOutOfArea() {
        const visibleWidth = 270;
        const contentLeftWith = this.editor.getOption(monaco.editor.EditorOption.layoutInfo).contentLeft;
        const minimapWith = this.editor.getOption(monaco.editor.EditorOption.layoutInfo).minimap.minimapWidth;
        if (this.editor.getLayoutInfo().width - contentLeftWith - minimapWith < visibleWidth) {
            return true;
        }
        return false;
    }
    setOptions(options) {
        this.options = options;
    }
    offsetTop(top) {
        if (this.originTop === 0) {
            this.originTop = this.domNode.style.top ? parseInt(this.domNode.style.top, 10) : 0;
        }
        this.domNode.style.top = `${this.originTop + top}px`;
    }
    id() {
        return index_1.AIInlineChatContentWidget;
    }
    getPosition() {
        const position = super.getPosition();
        if (position) {
            return position;
        }
        if (!this.options) {
            return null;
        }
        const { selection } = this.options;
        return selection ? this.computerPosition(selection) : null;
    }
    safeGetLineLastNonWhitespaceColumn(line) {
        const model = this.editor.getModel();
        return model.getLineLastNonWhitespaceColumn(Math.min(Math.max(1, line), model.getLineCount()));
    }
    toAbovePosition(lineNumber, column) {
        return {
            position: new monaco.Position(lineNumber, column),
            preference: [
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE,
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW,
            ],
        };
    }
    toBelowPosition(lineNumber, column) {
        return {
            position: new monaco.Position(lineNumber, column),
            preference: [
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW,
                browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE,
            ],
        };
    }
    recheckPosition(lineNumber, column) {
        const preNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber - 1);
        const curNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber);
        const nextNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(lineNumber + 1);
        let newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
        let newLineNumber = lineNumber;
        let newColumn = column;
        if (curNonWhitespaceColumn >= nextNonWhitespaceColumn) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
        }
        else if (curNonWhitespaceColumn >= preNonWhitespaceColumn) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
        }
        else {
            newColumn = Math.min(preNonWhitespaceColumn, nextNonWhitespaceColumn);
            if (preNonWhitespaceColumn >= nextNonWhitespaceColumn) {
                newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
                newLineNumber = lineNumber - 1;
            }
            else {
                newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.ABOVE];
                newLineNumber = lineNumber + 1;
            }
        }
        if (lineNumber === 1 || lineNumber === 2) {
            newPreference = [browser_1.monacoBrowser.editor.ContentWidgetPositionPreference.BELOW];
        }
        return {
            position: new monaco.Position(newLineNumber, newColumn),
            preference: newPreference,
        };
    }
    isProtrudeAbove(line) {
        const currentLastNonWhitespace = this.safeGetLineLastNonWhitespaceColumn(line);
        return (currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line - 1) &&
            currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line - 2));
    }
    isProtrudeBelow(line) {
        const currentLastNonWhitespace = this.safeGetLineLastNonWhitespaceColumn(line);
        return (currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line + 1) &&
            currentLastNonWhitespace >= this.safeGetLineLastNonWhitespaceColumn(line + 2));
    }
    /**
     * 动态计算要显示的位置
     * 1. 以选区里的光标作为顶点
     * 2. 靠近光标处周围没有字符的空白区域作为要显示的区域
     * 3. 显示的区域方向在右侧，左侧不考虑
     */
    computerPosition(selection) {
        const startPosition = selection.getStartPosition();
        const endPosition = selection.getEndPosition();
        const model = this.editor.getModel();
        if (!model) {
            return null;
        }
        const cursorPosition = selection.getPosition();
        const getCursorLastNonWhitespaceColumn = this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber);
        let targetLine = null;
        let direction = null;
        if (startPosition.lineNumber === endPosition.lineNumber) {
            return this.recheckPosition(cursorPosition.lineNumber, cursorPosition.column);
        }
        if (cursorPosition.equals(startPosition)) {
            const getMaxLastWhitespaceColumn = Math.max(this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber - 1), this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber - 2));
            if (getMaxLastWhitespaceColumn < getCursorLastNonWhitespaceColumn + 10) {
                return this.toAbovePosition(cursorPosition.lineNumber, getMaxLastWhitespaceColumn + 1);
            }
            for (let i = startPosition.lineNumber; i <= endPosition.lineNumber; i++) {
                if (this.isProtrudeAbove(i)) {
                    targetLine = i;
                    direction = 'above';
                    break;
                }
                if (this.isProtrudeBelow(i)) {
                    targetLine = i;
                    direction = 'below';
                    break;
                }
            }
        }
        else if (cursorPosition.equals(endPosition)) {
            const getMaxLastWhitespaceColumn = Math.max(this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber + 1), this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber + 2));
            if (getMaxLastWhitespaceColumn < getCursorLastNonWhitespaceColumn + 10) {
                return this.toBelowPosition(cursorPosition.lineNumber, getMaxLastWhitespaceColumn + 1);
            }
            for (let i = endPosition.lineNumber; i >= startPosition.lineNumber; i--) {
                if (this.isProtrudeBelow(i)) {
                    targetLine = i;
                    direction = 'below';
                    break;
                }
                if (this.isProtrudeAbove(i)) {
                    targetLine = i;
                    direction = 'above';
                    break;
                }
            }
        }
        if (targetLine && direction) {
            const column = this.safeGetLineLastNonWhitespaceColumn(targetLine) + 1;
            if (direction === 'below') {
                return this.toBelowPosition(targetLine, column);
            }
            return this.toAbovePosition(targetLine, column);
        }
        return this.recheckPosition(cursorPosition.lineNumber, this.safeGetLineLastNonWhitespaceColumn(cursorPosition.lineNumber));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AIInlineContentWidget.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAIInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AIInlineChatService)
], AIInlineContentWidget.prototype, "aiInlineChatService", void 0);
AIInlineContentWidget = tslib_1.__decorate([
    (0, di_1.Injectable)({ multiple: true }),
    tslib_1.__metadata("design:paramtypes", [Object])
], AIInlineContentWidget);
exports.AIInlineContentWidget = AIInlineContentWidget;
//# sourceMappingURL=inline-content-widget.js.map